@@ -53,7 +53,7 @@
 #![deny(missing_docs)]
 #![warn(noop_method_call)]
 #![deny(unreachable_pub)]
-#![deny(clippy::all)]
+#![warn(clippy::all)]
 #![deny(clippy::await_holding_lock)]
 #![deny(clippy::cargo_common_metadata)]
 #![deny(clippy::cast_lossless)]
@@ -88,11 +88,15 @@ mod err;
 pub mod types;
 mod util;
 
-pub use err::{Error, Pos};
+pub use err::{BuildError, Error, ParseErrorKind, Pos};
 
 /// Alias for the Result type returned by most objects in this module.
 pub type Result<T> = std::result::Result<T, Error>;
 
+/// Alias for the Result type returned by document-builder functions in this
+/// module.
+pub type BuildResult<T> = std::result::Result<T, BuildError>;
+
 /// Indicates whether we should parse an annotated list of objects or a
 /// non-annotated list.
 #[derive(PartialEq, Debug)]
